<?php
namespace JExtstore\Component\JRealtimeAnalytics\Administrator\Field;
/**  
 * @package JREALTIMEANALYTICS::CONFIG::components::com_jrealtimeanalytics
 * @subpackage Field
 * @author Joomla! Extensions Store
 * @copyright (C) 2013 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html   
 */ 
defined( '_JEXEC' ) or die( 'Restricted access' );
use Joomla\CMS\Form\Field\ListField;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

/**
 * Form Field for the extensions list
 * @package JREALTIMEANALYTICS::CONFIG::components::com_jrealtimeanalytics
 * @subpackage Field
 * @since 2.0
 */
class ExtensionsField extends ListField {
	/**
	 * The form field type.
	 *
	 * @var    string
	 *
	 * @since  11.1
	 */
	protected $type = 'extensions';
	
	/**
	 * Method to get the field input markup for a generic list.
	 * Use the multiple attribute to enable multiselect.
	 *
	 * @return string The field input markup.
	 *        
	 * @since 11.1
	 */
	protected function getInput() {
		// Initialize variables.
		$html = array ();
		$attr = '';
		
		// Initialize some field attributes.
		$attr .= $this->element ['class'] ? ' class="form-select ' . ( string ) $this->element ['class'] . '"' : ' class="form-select"';
		
		// To avoid user's confusion, readonly="true" should imply
		// disabled="true".
		if (( string ) $this->element ['readonly'] == 'true' || ( string ) $this->element ['disabled'] == 'true') {
			$attr .= ' disabled="disabled"';
		}
		
		$attr .= $this->element ['size'] ? ' size="' . ( int ) $this->element ['size'] . '"' : '';
		$attr .= $this->multiple ? ' multiple="multiple"' : '';
		
		// Initialize JavaScript field attributes.
		$attr .= $this->element ['onchange'] ? ' onchange="' . ( string ) $this->element ['onchange'] . '"' : '';
		
		// Get the field options.
		$options = ( array ) $this->getOptions ();
		
		$html = HTMLHelper::_ ( 'select.genericlist', $options, $this->name, trim ( $attr ), 'value', 'text', $this->value, $this->id );
		
		return $html;
	}
	
	/**
	 * Method to get the field options.
	 *
	 * @return  array  The field option objects.
	 *
	 * @since   11.1
	 */
	protected function getOptions() {
		$db = Factory::getContainer()->get('DatabaseDriver');
		$queryComponent = "SELECT DISTINCT " . $db->quoteName('element') . " AS value, SUBSTRING(" . $db->quoteName('element') . ", 5) AS text" .
						  "\n FROM #__extensions" .
						  "\n WHERE (" . $db->quoteName('protected') . " = 0" .
						  "\n OR " . $db->quoteName('element') . " = " . $db->quote('com_content') . // Exception for custom content sources
						  "\n OR " . $db->quoteName('element') . " = " . $db->quote('com_tags') . // Exception for custom tags sources
						  "\n OR " . $db->quoteName('element') . " = " . $db->quote('com_docman') . ")" . // Exception for docman 2 protected
		 				  "\n AND ". $db->quoteName('type') . " = " . $db->quote('component') .
		 				  "\n AND ". $db->quoteName('element') . " != " . $db->quote('com_jrealtimeanalytics') .
		 				  "\n ORDER BY text ASC";
		$db->setQuery($queryComponent);
		// Check for a database error.
		try {
			$options = $db->loadObjectList ();
		} catch(\Exception $e) {
			return array();
		}
		
		$noActiveOption = HTMLHelper::_('select.option', '0', Text::_('COM_JREALTIME_SELECT_EXTENSION'));
		array_unshift($options, $noActiveOption);
		
		return $options;
	}
}
